/*  Copyright (C) 2008 Niels Richard Hansen. 

    This file is part of StemSearch.

    StemSearch is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    StemSearch is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/ 

#include "StructSearch.h"
#include <stdlib.h>
#include <sstream>
#include <ctime>

void usage()
{
	cout << "\nStemSearch [-f sequence file] [-I start end] [-L bandwidth] [-t threshold] [-id] [-o s|nl] [-p parameter file] [-r nr of reported sequences] [-nome] [-nolp] [-l] [-g] [-s seed]\n\n";
	exit(1);
} 

int main(int argc, char* argv[]) 
{
	
  long i,j, start=0, end=0;
  stringstream arguments;
  char* filename = "";
  double threshold = 10;
  long band_size = 300;
  long seed = time(0);

  long nr=0;
  bool warn=false;
  bool SaveMatrices=false;
  bool greedy = false;
  enum method_value {ME,ID,LP,noME,noID,noLP};
  enum output {all,minimal,noloop};
  output out=all;
  method_value Mmethod = ME;	
  method_value Imethod = noID;
  method_value Lmethod = LP;
  string pfilename;

  if(argc == 1){usage();}

  for(i = 1; i < argc; i++)
    {
      arguments << argv[i] << " ";
      if(argv[i][0] == '-')
	{
	  switch (argv[i][1])
	    {
	    case 'f':
	      if (i == argc-1) usage();
	      filename = argv[i + 1];
	      break;
	    case 'p':
	      if (i == argc-1) usage();
	      pfilename = argv[i + 1];
	      break;
	    case 't':
	      if (i == argc-1) usage();
	      threshold = atof(argv[i + 1]);
	      break;
	    case 'L':
	      if (i == argc-1) usage();
	      band_size = atoi(argv[i + 1]);
	      break;
	    case 'b': //old use - for compatibility
	      if (i == argc-1) usage();
	      band_size = atoi(argv[i + 1]);
	      break;
	    case 'I':
	      if ((i == argc-2) | (i==argc-1)) usage();
	      start = atoi(argv[i + 1]);
	      end = atoi(argv[i + 2]);
	      break;
	    case 'r':
	      if (i == argc-1) usage();
	      nr = atoi(argv[i + 1]);
	      break;
	    case 'n':
	      switch (argv[i][3])
		{
		case 'm':
		  Mmethod = noME;
		  break;
		case 'l':
		  Lmethod = noLP;
		  break;
		} 
	      break;
	    case 'i': 
	      Imethod=ID;
	      break;
	    case 'l':
	      warn=true;
	      break;
	    case 's':
	      if (i == argc-1) usage();
	      seed = atoi(argv[i + 1]);
	      break;	
	    case 'g':
	      greedy=true;
	      break;
	    case 'o':
	      if (i == argc-1) usage();
	      switch (argv[i+1][0])
		{
		case 's':
		  out=minimal;
		  break;
		case 'n':
		  out=noloop;
		  break;
		}
	      break;
	    }
	}
    }	
	
	
	if(SaveMatrices) 
	{
		Mmethod = noME;
		Imethod = ID;
	}
	
	if(greedy)
	{
		Mmethod = noME;
		Imethod = ID;
		Lmethod = noLP;
	}
	
	if(warn)
	  {
	    ofstream file("StemSearch.log", ios_base::app);
	    time_t rawtime = time(NULL);
	    string logtime = ctime(&rawtime);
	    file << logtime.substr(0,24) << "\t\tSeed:\t" << seed << "\n";
	    file.close();
	  } 
	cout << "#StemSearch " << arguments.str()  << "\n";
	if(out==minimal) {cout << "Seq.id \t Pos.i \t Pos.j \t Anc.i \t Anc.j \t score \t E.value\n";}
	StructSearch StructObject(band_size,threshold,pfilename.c_str(),(Lmethod == LP),warn,start,end);
	ifstream tmpstream(filename);

	while(!StructObject.done())
	{
	  if(tmpstream) StructObject.read(tmpstream,'>');
	  else StructObject.generate(seed);

		switch (Mmethod)
		{
			case ME:
				switch (Imethod)
				{
					case ID:
						StructObject.computeESIT();
					break;
					case noID:
						StructObject.computeEST();
					break;
				}
			break;
			case noME: 
				switch (Imethod)
				{
					case ID:
						StructObject.computeSIT();
					break;
					case noID:
						StructObject.computeST();
					break;
				}
			break;
		}
	
 	StructObject.traceback();
	if(SaveMatrices) StructObject.saveI();
	if(greedy) StructObject.greedystr();
	
	cout << StructObject.pretty_print(nr,out);
	
	}
		
}
