/*  Copyright (C) 2007 Niels Richard Hansen. 

    This file is part of StemSearch.

    StemSearch is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    StemSearch is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "random.h"

uni::uni(long k)
	{   
	
		IA = 16807;
		IM = 2147483647;
		AM = 1.0/((double) IM);
		IQ = 127773;
		IR = 2836;
		NTAB = 32;
		NDIV = 1 + (IM-1)/NTAB;
		EPS = 1.2e-7; 
		RNMX = 1.0 - EPS;
		v = new long[NTAB];
		iv =0;
		setseed(k);
		iv = v[0];
	}

double uni::next()
{
	long tmp; 
	
	tmp = current/IQ;
	current = IA*(current - tmp*IQ) - IR*tmp;
	if (current < 0) current += IM;
	
	tmp = iv/NDIV;
	iv = v[tmp];
	v[tmp] = current;

	return AM*iv;
}
	
void uni::setseed(long k) 
{
  	int j;
	long tmp;
  	
	if (k > 0) seed = k; else seed = 1;
	current = seed;
	for (j=NTAB+7; j>=0; j--)
	  {
	    tmp = current/IQ;
	    current = IA*(current - tmp*IQ) - IR*tmp;
	    if (current < 0) current += IM;     
	    if (j < NTAB) v[j] = current;
	  }
}




